@@ -1,5 +1,5 @@
 import warnings
-from typing import List
+from typing import List, Optional, cast
 
 import torch
 import torch.nn as nn
@@ -41,12 +41,14 @@ class Buddy(
             the first call to minimize(). Set to 0 to disable. Defaults to 300.
         verbose (bool, optional): Flag for toggling debug messages. Defaults to
             `True`.
+        cpu_only (bool, optional): Set to True to turn off auto-detection of cuda
+            support.
     """
 
     def __init__(
         self,
         experiment_name: str,
-        model: nn.Module,
+        model: Optional[nn.Module] = None,
         *,
         checkpoint_dir: str = "checkpoints",
         checkpoint_max_to_keep: int = 5,
@@ -54,28 +56,32 @@ class Buddy(
         log_dir: str = "logs",
         optimizer_type: str = "adam",
         optimizer_checkpoint_interval: float = 300,
-        optimizer_names: List[str] = None,  # Deprecated!
+        optimizer_names: Optional[List[str]] = None,  # Deprecated!
         verbose: bool = True,
+        cpu_only: bool = False,
     ) -> None:
         """Constructor
         """
         # Validate and assign core parameters.
         assert type(experiment_name) == str
-        assert isinstance(model, nn.Module)
         assert type(verbose) == bool
 
         self._experiment_name = experiment_name
-        self._model = model
         self._verbose = True
 
         # Use GPU for training if available.
-        if torch.cuda.is_available():
+        if not cpu_only and torch.cuda.is_available():
             self._device = torch.device("cuda")
-            model.cuda()
         else:
             self._device = torch.device("cpu")
         self._print("Using device:", self._device)
 
+        # Attach model if available
+        if model != None:
+            self.attach_model(cast(nn.Module, model))
+        else:
+            self._model = None
+
         # Call constructors for each of our interfaces.
         # Sets up checkpointing, metadata, logging, and optimization-specific state.
         #
@@ -96,10 +102,30 @@ class Buddy(
         # Print available checkpoints
         self._print("Available checkpoint labels:", self.checkpoint_labels)
 
+    def attach_model(self, model: nn.Module) -> None:
+        """Attach a model to our Buddy, and move it onto `buddy.device`.
+
+        If a model isn't explicitly passed into the constructor's `model` field,
+        `attach_model` should be called before any optimization, checkpointing, etc
+        happens.
+
+        Args:
+            model (nn.Module): Model to attach.
+        """
+
+        assert isinstance(model, nn.Module)
+
+        # Move model to correct device
+        model.to(self._device)
+
+        # Attach model
+        self._model = model
+
     # Shared functions
     @property
     def device(self) -> torch.device:
-        """Read-only interface for the active torch device.
+        """Read-only interface for the active torch device. Auto-detected in the
+        constructor based on CUDA support.
         """
         return self._device
 
