@@ -50,6 +50,11 @@ class _BuddyOptimizer(_BuddyForwardDeclarations, abc.ABC):
         self._optimizer_checkpoint_interval: float = optimizer_checkpoint_interval
         self._optimizer_last_checkpoint_time: Optional[float] = None
 
+        # Default learning rate
+        self._optimizer_default_learning_rate: Union[
+            float, Callable[[int], float]
+        ] = self._OPTIMIZER_DEFAULT_LEARNING_RATES[optimizer_type]
+
     def minimize(
         self,
         loss: torch.Tensor,
@@ -60,7 +65,7 @@ class _BuddyOptimizer(_BuddyForwardDeclarations, abc.ABC):
     ) -> None:
         """Compute gradients and use them to minimize a loss function.
         """
-
+        assert self._model is not None, "No model attached!"
         self._instantiate_optimizer(optimizer_name)
 
         # Update learning rate using scheduler if possible
@@ -101,6 +106,7 @@ class _BuddyOptimizer(_BuddyForwardDeclarations, abc.ABC):
     def get_learning_rate(self, optimizer_name: str = "primary") -> float:
         """Gets an optimizer learning rate.
         """
+        assert self._model is not None, "No model attached!"
         assert optimizer_name in self._optimizer_dict
 
         # Return scheduled learning rate
@@ -122,7 +128,7 @@ class _BuddyOptimizer(_BuddyForwardDeclarations, abc.ABC):
         """Sets an optimizer learning rate. Accepts either a floating point
         learning rate or a schedule function (int steps -> float LR).
         """
-
+        assert self._model is not None, "No model attached!"
         schedulers = self._optimizer_config["learning_rate_schedulers"]
         if callable(value):
             # Store a scheduler
@@ -138,6 +144,13 @@ class _BuddyOptimizer(_BuddyForwardDeclarations, abc.ABC):
             # Set scalar learning rate
             self._set_learning_rate(value, optimizer_name)
 
+    def set_default_learning_rate(
+        self, value: Union[float, Callable[[int], float]]
+    ) -> None:
+        """Sets a default learning rate for new optimizers.
+        """
+        self._optimizer_default_learning_rate = value
+
     @property
     def optimizer_steps(self) -> int:
         """Read-only interface for # of steps taken by optimizer.
@@ -159,6 +172,7 @@ class _BuddyOptimizer(_BuddyForwardDeclarations, abc.ABC):
         """(Private) Instantiates an optimizer. Returns immediately if
         optimizer already exists.
         """
+        assert self._model is not None, "No model attached!"
         if optimizer_name in self._optimizer_dict.keys():
             # Optimizer already exists: do nothing!
             return
@@ -171,7 +185,7 @@ class _BuddyOptimizer(_BuddyForwardDeclarations, abc.ABC):
 
         # Parameters
         Optimizer = self._OPTIMIZER_TYPES[optimizer_type]
-        initial_learning_rate = self._OPTIMIZER_DEFAULT_LEARNING_RATES[optimizer_type]
+        initial_learning_rate = self._optimizer_default_learning_rate
 
         # Construct optimizer
         self._optimizer_dict[optimizer_name] = Optimizer(
