@@ -5,6 +5,7 @@ from typing import (
     Callable,
     Dict,
     Generic,
+    Iterable,
     List,
     Tuple,
     TypeVar,
@@ -21,16 +22,26 @@ import fannypack
 # Valid raw types that we can wrap
 _raw_types = set([list, tuple, np.ndarray, torch.Tensor])
 
-# Alias for dictionary types
-DictType = Union[
-    Dict[Any, List], Dict[Any, Tuple], Dict[Any, torch.Tensor], Dict[Any, np.ndarray],
-]
-
-# Alias for valid raw types that we can wrap
-InputType = Union[List, Tuple, torch.Tensor, np.ndarray, DictType]
+# Generic types
+MapOutputType = TypeVar("MapOutputType")
+WrappedType = TypeVar(
+    "WrappedType",
+    List,
+    Tuple,
+    torch.Tensor,
+    np.ndarray,
+    Dict[Any, List],
+    Dict[Any, Tuple],
+    Dict[Any, torch.Tensor],
+    Dict[Any, np.ndarray],
+    Union[List, Dict[Any, List]],
+    Union[Tuple, Dict[Any, Tuple]],
+    Union[torch.Tensor, Dict[Any, torch.Tensor]],
+    Union[np.ndarray, Dict[Any, np.ndarray]],
+)
 
 
-class SliceWrapper:
+class SliceWrapper(Iterable, Generic[WrappedType]):
     """A thin wrapper class for creating a unified interface for...
     - Lists
     - Tuples
@@ -43,8 +54,8 @@ class SliceWrapper:
     calling `append` and `extend` on wrapped lists.
     """
 
-    def __init__(self, data: InputType):
-        self.data = data
+    def __init__(self, data: WrappedType):
+        self.data: WrappedType = data
         """list, tuple, torch.Tensor, np.ndarray, or dict: Wrapped data."""
 
         # Sanity checks
@@ -63,11 +74,11 @@ class SliceWrapper:
                 assert content_type in _raw_types
         else:
             # Non-dictionary inputs
-            assert type(data) in _raw_types, "Invalid datatype!"
+            assert type(data) in _raw_types, "Unsupported datatype!"
 
         # Backwards-compatibility
         def convert_to_numpy():  # pragma: no cover
-            self.data = self.map(np.asarray).data
+            self.data = self.map(np.asarray)
 
         self.convert_to_numpy = fannypack.utils.deprecation_wrapper(
             "SliceWrapper.convert_to_numpy() is deprecated -- please use "
@@ -76,7 +87,8 @@ class SliceWrapper:
         )
 
     def __getitem__(self, index: Any) -> Any:
-        """Unified interface for indexing into our wrapped object.
+        """Unified interface for indexing into our wrapped object; shorthand for
+        `SliceWrapper.get(lambda v: v[index])`.
 
         For iterables that are directly wrapped, this is equivalent to evaluating
         `data[index]`.
@@ -104,24 +116,7 @@ class SliceWrapper:
         Returns:
             Any: Indexed value. See overall function docstring.
         """
-        if type(self.data) == dict:
-            # Cast for type-checking
-            data_dict = cast(dict, self.data)
-
-            # Check that the index is sane
-            # Allows use as a standard iterator, eg `for obj in SliceWrapper(...)`
-            if type(index) == int and index >= len(self):
-                raise IndexError
-
-            # Construct & return output
-            output = {}
-            for key, value in data_dict.items():
-                output[key] = value[index]
-            return output
-        elif type(self.data) in _raw_types:
-            return self.data[index]
-        else:
-            assert False, "Invalid operation!"
+        return cast(WrappedType, self.map(lambda v: v[index]))
 
     def __len__(self) -> int:
         """Unified interface for evaluating the length of a wrapped object.
@@ -133,6 +128,23 @@ class SliceWrapper:
         """
         return self.shape[0]
 
+    def __iter__(self):
+        """Iterable __iter__() interface.
+        """
+        self._iter_index = 0
+        return self
+
+    def __next__(self):
+        """Iterable __next__() interface.
+        """
+        try:
+            output = self[self._iter_index]
+            self._iter_index += 1
+            return output
+        except IndexError:
+            pass
+        raise StopIteration
+
     def append(self, other: Any) -> None:
         """Append to the end of our data object.
 
@@ -176,7 +188,7 @@ class SliceWrapper:
         else:
             assert False, "Append is only supported for wrapped lists"
 
-    def extend(self, other: InputType) -> None:
+    def extend(self, other: WrappedType) -> None:
         """Extend to the end of our data object.
 
         Only supported for wrapped lists and dictionaries containing lists.
@@ -219,13 +231,15 @@ class SliceWrapper:
         else:
             assert False, "Extend is only supported for wrapped lists"
 
-    def map(self, function: Callable[[Any], Any]) -> SliceWrapper:
+    def map(
+        self, function: Callable[[Any], MapOutputType]
+    ) -> Union[MapOutputType, Dict[Any, MapOutputType]]:
         """Compute a new SliceWrapper, with a function applied to all values within
         our wrapped data object.
 
         For iterables that are directly wrapped, this is equivalent to evaluating:
         ```
-        SliceWrapper(function(data))
+        function(data)
         ```
 
         For dictionaries, `function` is applied value-wise.
@@ -238,10 +252,10 @@ class SliceWrapper:
         ```
         would return...
         ```
-        SliceWrapper({
+        {
             "a": function(a),
             "b": function(b),
-        })
+        }
         ```
 
         Args:
@@ -252,19 +266,20 @@ class SliceWrapper:
             data_dict = cast(dict, self.data)
 
             # Construct output
-            mapped_data_dict = {}
+            mapped_data = {}
             for key, value in data_dict.items():
-                mapped_data_dict[key] = function(value)
+                mapped_data[key] = function(value)
+            return mapped_data
+        elif type(self.data) in _raw_types:
+            return function(self.data)
         else:
-            mapped_data_dict = function(self.data)
-
-        return SliceWrapper(mapped_data_dict)
+            assert False, f"Unsupported data type: {type(self.data)}"
 
     @property
     def shape(self) -> Tuple[int, ...]:
         """Unified interface for polling the shape of our wrapped object.
 
-        For lists and tuples, this simply evaluates to `(len(data),)`.
+        For lists and tuples, this evaluates to `(len(data),)`.
 
         For Numpy arrays and torch tensors, we get `data.shape`.
 
@@ -296,8 +311,10 @@ class SliceWrapper:
                         break
 
             return tuple(output)
-        else:
+        elif type(self.data) in _raw_types:
             return self._shape_helper(self.data)
+        else:
+            assert False, "Unsupported datatype!"
 
     @staticmethod
     def _shape_helper(data) -> Tuple[int, ...]:
@@ -311,4 +328,4 @@ class SliceWrapper:
             # Return 1D shape
             return (len(data),)
         else:
-            assert False, "Invalid operation!"
+            assert False, "Unsupported datatype!"
