@@ -10,6 +10,22 @@ import fannypack
 from ..fixtures import simple_buddy_temporary_data
 
 
+def test_buddy_no_model():
+    """Check that errors are raised if a Buddy is used without a model attached.
+    """
+    buddy = fannypack.utils.Buddy("no_model")
+
+    with pytest.raises(AssertionError):
+        buddy.save_checkpoint()
+
+    with pytest.raises(AssertionError):
+        buddy.load_checkpoint()
+
+    with pytest.raises(AssertionError):
+        fake_loss = torch.Tensor([0.0]).to(buddy.device)
+        buddy.minimize(fake_loss)
+
+
 def test_buddy_log_scopes(simple_buddy_temporary_data):
     """Check that log scope function as expected.
     """
@@ -48,8 +64,8 @@ def test_buddy_train(simple_buddy_temporary_data):
 
     initial_loss = F.mse_loss(model(data), labels)
 
-    # Try using LR scheduler interface
-    buddy.set_learning_rate(lambda steps: 1e-3)
+    # Try using default learning rate
+    buddy.set_default_learning_rate(lambda steps: 1e-3)
 
     for _ in range(200):
         # Optimize
@@ -60,6 +76,7 @@ def test_buddy_train(simple_buddy_temporary_data):
         with buddy.log_scope("scope"):
             buddy.log_scalar("loss", loss)
 
+    assert buddy.get_learning_rate() == 1e-3
     assert buddy.optimizer_steps == 200
 
     # Loss should at least have halved
